#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  epubtweak
#  Copyright © 2009  Mikael Ylikoski
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
#  epubtweak lets you modify certain values in the OPF file of an EPUB file.
#

import optparse
import os
import uuid
import xml.dom.minidom
import zipfile

dc_ns = "http://purl.org/dc/elements/1.1/"
ocf_ns = "urn:oasis:names:tc:opendocument:xmlns:container"
opf_ns = "http://www.idpf.org/2007/opf"

version = "epubtweak 0.3"
usage = "Usage: %prog [options] <EPUB file>"
parser = optparse.OptionParser(usage = usage, version = version)
parser.add_option("--publisher", action = "store", type = "string",
                  dest = "publisher", help = "set publisher string")
parser.add_option("--uid", action = "store", type = "string",
                  dest = "uid", help = "set unique identifier string")
parser.add_option("--uuid", action = "store_true", dest = "uuid",
                  help = "set unique identifier string to random uuid")

(options, args) = parser.parse_args()
if len(args) != 1:
    parser.error("incorrect number of arguments")

epubfn = args[0]

def get_opf(epub):
    zf = zipfile.ZipFile(epub, "r")

    opfname = None
    contfile = zf.open("META-INF/container.xml")
    cont = xml.dom.minidom.parse(contfile)
    contfile.close()
    for el in cont.getElementsByTagNameNS(ocf_ns, "rootfile"):
        if el.hasAttribute("media-type") and el.hasAttribute("full-path") and \
                el.getAttribute("media-type") == "application/oebps-package+xml":
            opfname = el.getAttribute("full-path")
            break

    if not opfname:
        return (None, None)

    opffile = zf.open(opfname)
    opf = xml.dom.minidom.parse(opffile)
    opffile.close()
    return (opfname, opf)

(opfname, opf) = get_opf(epubfn)
if not opf:
    print("Error: Cannot find OPF")
    exit(1)

metadata = opf.getElementsByTagNameNS(opf_ns, "metadata")[0]

def remove_children(el):
    while el.firstChild:
        el.removeChild(el.firstChild)

if options.uuid or options.uid:
    el = opf.getElementsByTagNameNS(opf_ns, "package")[0]
    uiid = el.getAttribute("unique-identifier")

    idel = None
    for ide in metadata.getElementsByTagNameNS(dc_ns, "identifier"):
        if ide.getAttribute("id") == uiid:
            idel = ide
            break
    if not idel:
        print("Error: Cannot find unique identifier")
        exit(1)

    remove_children(idel)

    uid = None
    if options.uuid:
        uid = "urn:uuid:" + str(uuid.uuid4())
    else:
        uid = options.uid
    print("Setting unique identifier to " + uid)
    text = opf.createTextNode(uid)
    idel.appendChild(text)

if options.publisher:
    print("Setting publisher to " + options.publisher)
    el = metadata.getElementsByTagNameNS(dc_ns, "publisher")
    if el:
        remove_children(el)
    else:
        el = opf.createElementNS(dc_ns, "dc:publisher")
        el.setAttribute("xmlns:dc", dc_ns)
        metadata.appendChild(el)
    text = opf.createTextNode(options.publisher)
    el.appendChild(text)

xmlstr = opf.toxml("utf-8")

os.system("zip " + epubfn + " -d " + opfname)
zf = zipfile.ZipFile(epubfn, "a", zipfile.ZIP_DEFLATED)
zf.writestr(opfname, xmlstr)
zf.close()

# newzf = zipfile.ZipFile(epubfn + ".new", "w", zipfile.ZIP_DEFLATED)
# newzf.writestr("mimetype", "application/epub+zip")
# newzf.writestr(opfname, xmlstr)

# for name in zf.namelist():
#     if name != opffile and name != "MIMETYPE":
#         fil = zf.open(name)
#         newzf.write(fil, name)
# zf.close()
